@@ -4,7 +4,7 @@
 ##' Langevin algorithm.
 ##'
 ##' @param formula the model formula in a format compatible with the function flexsurvreg from the flexsurv package 
-##' @param data a SpatialPointsDataFrame object containing the survival data as one of the columns
+##' @param data a SpatialPointsDataFrame object containing the survival data as one of the columns OR for polygonal data a data.frame, in which case, the argument shape must also be supplied
 ##' @param dist choice of distribution function for baseline hazard. Current options are: exponentialHaz, weibullHaz, gompertzHaz, makehamHaz, tpowHaz
 ##' @param cov.model an object of class covmodel, see ?covmodel ?ExponentialCovFct or ?SpikedExponentialCovFct
 ##' @param mcmc.control mcmc control parameters, see ?mcmcpars
@@ -12,6 +12,7 @@
 ##' @param shape when data is a data.frame, this can be a SpatialPolygonsDataFrame, or a SpatialPointsDataFrame, used to model spatial variation at the small region level. The regions are the polygons, or they represent the (possibly weighted) centroids of the polygons.
 ##' @param ids named list entry shpid character string giving name of variable in shape to be matched to variable dataid in data. dataid is the second entry of the named list.
 ##' @param control additional control parameters, see ?inference.control
+##' @param boundingbox optional bounding box over which to construct computational grid, supplied as an object on which the function 'bbox' returns the bounding box
 ##' @return an object inheriting class 'mcmcspatsurv' for which there exist methods for printing, summarising and making inference from.
 ##' @seealso \link{tpowHaz}, \link{exponentialHaz}, \link{gompertzHaz}, \link{makehamHaz}, \link{weibullHaz}, 
 ##' \link{covmodel}, link{ExponentialCovFct}, \code{SpikedExponentialCovFct},
@@ -31,7 +32,12 @@ survspat <- function(   formula,
                         priors,
                         shape=NULL,
                         ids=list(shpid=NULL,dataid=NULL),
-                        control=inference.control(gridded=FALSE)){
+                        control=inference.control(gridded=FALSE),
+                        boundingbox=NULL){
+
+    if(!is.null(boundingbox)){
+        boundingbox <- spTransform(boundingbox,CRS(proj4string(data)))
+    }
                         
     formula <- as.formula(formula)                    
     
@@ -46,6 +52,9 @@ survspat <- function(   formula,
     latentmode <- "points"
     if(inherits(data,"data.frame")){
         latentmode <- "polygons"
+        if(!is.null(control$imputation)){
+            latentmode <- "points"
+        }
     }
     if(inherits(cov.model,"SPDEmodel")){
         latentmode <- "SPDE"
@@ -53,7 +62,7 @@ survspat <- function(   formula,
 
     if(inherits(data,"SpatialPointsDataFrame")&!is.null(shape)){
         if(latentmode!="SPDE"){
-            warning("Non NULL shape, ignoring shape.",immediate.=TRUE)
+            warning("Non NULL shape with spatial points survival data, are you sure you want to proceed ???",immediate.=TRUE)
             Sys.sleep(2)
         }
     }
@@ -68,7 +77,12 @@ survspat <- function(   formula,
 
     responsename <- as.character(formula[[2]])
     if(latentmode=="points"){
-        survivaldata <- data@data[[responsename]]
+        if(!is.null(control$imputation)){
+            survivaldata <- data@data[[responsename]]
+        }
+        else{
+            survivaldata <- data[[responsename]]
+        }
     }
     else{
         survivaldata <- data[[responsename]]
@@ -76,9 +90,12 @@ survspat <- function(   formula,
     checkSurvivalData(survivaldata) 
 
     if(latentmode=="SPDE"){
-        if(proj4string(data)!=proj4string(shape)){
-            stop("'shape' and 'data' must have the same proj4string.")
-        } 
+        if(!is.na(proj4string(data)) & !is.na(proj4string(shape))){
+            if(proj4string(data)!=proj4string(shape)){
+                stop("'shape' and 'data' must have the same proj4string.")
+            }
+        }
+
         if(is.null(control$cellwidth)){
             stop("Must specify 'cellwidth' in inference.control in SPDE mode.")
         }   
@@ -119,7 +136,7 @@ survspat <- function(   formula,
     gridobj <- NULL
    
     if(control$gridded){
-        gridobj <- FFTgrid(spatialdata=data,cellwidth=control$cellwidth,ext=control$ext)
+        gridobj <- FFTgrid(spatialdata=data,cellwidth=control$cellwidth,ext=control$ext,boundingbox=boundingbox)
     	del1 <- gridobj$del1
     	del2 <- gridobj$del2
     	Mext <- gridobj$Mext
@@ -297,6 +314,10 @@ survspat <- function(   formula,
 
     #######
 
+    if(control$nugget){
+        control$U <- 0
+    }
+
     
     cat("\n","Getting initial estimates of model parameters using maximum likelihood on non-spatial version of the model","\n")
     mlmod <- maxlikparamPHsurv(surv=survivaldata,X=X,control=control)
@@ -323,7 +344,14 @@ survspat <- function(   formula,
                         surv=survivaldata,
                         control=control)
                         
-    calibrate <- get(paste("proposalVariance",funtxt,sep=""))    
+    calibrate <- get(paste("proposalVariance",funtxt,sep=""))   
+
+    if(control$nugget){ # dummy for calibration purposes
+        control$U <- rep(0,nrow(X))
+        control$Ugamma <- rep(0,nrow(X))
+        control$Usigma <- 0
+        control$logUsigma <- 0
+    } 
        
     other <- calibrate( X=X,
                         surv=survivaldata,
@@ -334,6 +362,8 @@ survspat <- function(   formula,
                         cov.model=cov.model,
                         u=u,
                         control=control) 
+
+
     
     #gammahat <- other$gammahat
     etahat <- other$etahat                                                                        
@@ -359,6 +389,8 @@ survspat <- function(   formula,
     
     
     npars <- lenbeta + lenomega + leneta + lengamma
+
+    #SIGMA <- diag(1e-4,npars)
     
     SIGMA[1:(lenbeta+lenomega),1:(lenbeta+lenomega)] <- (1.65^2/((lenbeta+lenomega)^(1/3)))*SIGMA[1:(lenbeta+lenomega),1:(lenbeta+lenomega)]
     SIGMA[(lenbeta+lenomega+1):(lenbeta+lenomega+leneta),(lenbeta+lenomega+1):(lenbeta+lenomega+leneta)] <- 0.4*(2.38^2/leneta)* SIGMA[(lenbeta+lenomega+1):(lenbeta+lenomega+leneta),(lenbeta+lenomega+1):(lenbeta+lenomega+leneta)]
@@ -368,28 +400,65 @@ survspat <- function(   formula,
         matidx <- matrix(0,control$Mext,control$Next)
         matidx[1:(control$Mext/control$ext),1:(control$Next/control$ext)] <- 1
         matidx <- as.logical(matidx) # used to select which Y's to save 
-    }
-  
-    
+    } 
+
     diagidx <- 1:npars
     diagidx <- matrix(diagidx,nrow=npars,ncol=2)
-    SIGMApars <- as.matrix(SIGMA[1:(lenbeta+lenomega+leneta),1:(lenbeta+lenomega+leneta)])
-    
+
+    SIGMApars <- as.matrix(SIGMA[1:(lenbeta+lenomega+leneta),1:(lenbeta+lenomega+leneta)])    
+    if(any(eigen(SIGMApars)$values<0)){
+        SIGMApars <- as.matrix(nearPD(SIGMApars)$mat)
+        warning("Fixing parameter proposal matrix ... maybe check model priors?",immediate.=TRUE)
+    }
+
     SIGMAparsINV <- solve(SIGMApars)
-    cholSIGMApars <- t(chol(SIGMApars))  
+    cholSIGMApars <- t(chol(SIGMApars)) 
   
     SIGMAgamma <- SIGMA[diagidx][(lenbeta+lenomega+leneta+1):npars]
     SIGMAgammaINV <- 1/SIGMAgamma
     cholSIGMAgamma <- sqrt(SIGMAgamma)
     
-     
-    
+    if(control$nugget){
+
+        Ugamma <- rep(0,nrow(X)) 
+        control$Ugamma <- Ugamma 
+
+        #SIGMAgamma <- control$split * SIGMA[diagidx][(lenbeta+lenomega+leneta+1):npars]
+        #SIGMAgammaINV <- 1/SIGMAgamma
+        #cholSIGMAgamma <- sqrt(SIGMAgamma)
+
+        #SIGMA_Ugamma <- mean((1-control$split) * SIGMA[diagidx][(lenbeta+lenomega+leneta+1):npars]) # proposal variance for Ugamma
+        SIGMA_Ugamma <- mean(SIGMA[diagidx][(lenbeta+lenomega+leneta+1):npars])
+        SIGMA_UgammaINV <- 1/SIGMA_Ugamma
+        cholSIGMA_Ugamma <- sqrt(SIGMA_Ugamma)
+
+        etatemp <- cov.model$itrans(eta)
+        #Usigma <- (1-control$split) * etatemp[control$sigmaidx] # initialise Usigma where U = Usigma * Ugamma
+        #Usigma <- etatemp[control$sigmaidx]
+        Usigma <- exp(control$logUsigma_priormean)
+        logUsigma <- log(Usigma)
+        #etatemp[control$sigmaidx] <- control$split * etatemp[control$sigmaidx]
+        #eta <- cov.model$trans(etatemp)
+
+        #SIGMA_logUsigma <- (1-control$split) * SIGMA[(lenbeta+lenomega+1):(lenbeta+lenomega+leneta),(lenbeta+lenomega+1):(lenbeta+lenomega+leneta)][control$sigmaidx,control$sigmaidx]
+        SIGMA_logUsigma <- SIGMA[(lenbeta+lenomega+1):(lenbeta+lenomega+leneta),(lenbeta+lenomega+1):(lenbeta+lenomega+leneta)][control$sigmaidx,control$sigmaidx]
+        SIGMA_logUsigmaINV <- 1/SIGMA_logUsigma
+        cholSIGMA_logUsigma <- sqrt(SIGMA_logUsigma)
+
+        control$U <- -Usigma^2/2 # U this will be updated using a random walk, not worrying about approximately optimal scaling
+        control$Usigma <- Usigma
+        control$logUsigma <- logUsigma
+
+    }
+
+    #browser()
+
+
     cat("Running MCMC ...\n")
     
     h <- 1
     
     
-    
     LOGPOST <- get(paste("logPosterior",funtxt,sep=""))
     
     oldlogpost <- LOGPOST(  surv=survivaldata,
@@ -403,8 +472,11 @@ survspat <- function(   formula,
                             u=u,
                             control=control,
                             gradient=TRUE)
-                            
-                          
+    
+    if(control$nugget){                        
+        oldUprior <- sum(dnorm(control$Ugamma,log=TRUE))
+        oldUsigmaprior <- dnorm(control$logUsigma,control$logUsigma_priormean,control$logUsigma_priorsd,log=TRUE)                      
+    }
                                                         
     
     betasamp <- c()
@@ -414,7 +486,13 @@ survspat <- function(   formula,
     
     gamma <- c(gamma) # turn gamma into a vector 
     
-    tarrec <- oldlogpost$logpost
+    if(control$nugget){
+        tarrec <- oldlogpost$logpost + oldUprior + oldUsigmaprior
+    }
+    else{
+        tarrec <- oldlogpost$logpost 
+    }
+    
     
     print(SIGMA[1:8,1:8])
     
@@ -430,6 +508,16 @@ survspat <- function(   formula,
 
     bad <-  c()
     
+    #gammasave <- c()
+
+    indiv_loglik <- c()
+    Umean <- 0
+    Uvar <- 0
+    Usigma_mean <- 0
+    Usigma_var <- 0
+
+    U_save <- c()
+    Usigma_save <- c()
     
     while(nextStep(mcmcloop)){
 
@@ -444,6 +532,25 @@ survspat <- function(   formula,
         if(control$gridded){
             ngam <- matrix(ngam,control$Mext,control$Next)
         }
+
+        if(control$nugget){
+            oldU <- control$U
+            oldUgamma <- control$Ugamma
+            oldUsigma <- control$Usigma
+            oldlogUsigma <- control$logUsigma
+
+            propmeanUgamma <- oldUgamma + (h/2)*SIGMA_Ugamma*oldlogpost$dP_dUgamma
+            newUgamma <- propmeanUgamma + sqrt(h)*cholSIGMA_Ugamma*rnorm(nrow(X))
+            
+            propmeanlogUsigma <- oldlogUsigma + (h/2)*SIGMA_logUsigma*oldlogpost$dP_dlogUsigma
+            newlogUsigma <- propmeanlogUsigma + sqrt(h)*cholSIGMA_logUsigma*rnorm(1)
+            
+            control$U <- -exp(newlogUsigma)^2/2 + exp(newlogUsigma) * newUgamma
+            control$Ugamma <- newUgamma
+            control$Usigma <- exp(newlogUsigma)
+            control$logUsigma <- newlogUsigma
+
+        }
                                        
         newlogpost <- LOGPOST(  surv=survivaldata,
                                 X=X,
@@ -458,7 +565,12 @@ survspat <- function(   formula,
                                 gradient=TRUE)
 
         revmeanpars <- newstuffpars + (h/2)*SIGMApars%*%newlogpost$grad[1:(lenbeta+lenomega+leneta)]
-        revmeangamma <- newstuffgamma + (h/2)*SIGMAgamma*newlogpost$grad[(lenbeta+lenomega+leneta+1):npars]       
+        revmeangamma <- newstuffgamma + (h/2)*SIGMAgamma*newlogpost$grad[(lenbeta+lenomega+leneta+1):npars]
+
+        if(control$nugget){
+            revmeanUgamma <- newUgamma + (h/2)*SIGMA_Ugamma*newlogpost$dP_dUgamma
+            revmeanlogUsigma <- newlogUsigma + (h/2)*SIGMA_logUsigma*newlogpost$dP_dlogUsigma       
+        }
 
         revdiffpars <- as.matrix(stuffpars-revmeanpars)
         forwdiffpars <- as.matrix(newstuffpars-propmeanpars)
@@ -471,6 +583,24 @@ survspat <- function(   formula,
                             (0.5/h)*t(forwdiffpars)%*%SIGMAparsINV%*%forwdiffpars -
                             (0.5/h)*sum(revdiffgamma*SIGMAgammaINV*revdiffgamma) + 
                             (0.5/h)*sum(forwdiffgamma*SIGMAgammaINV*forwdiffgamma)
+
+        if(control$nugget){
+
+            revdifflogUsigma <- as.matrix(oldlogUsigma-revmeanlogUsigma)
+            forwdifflogUsigma <- as.matrix(control$logUsigma-propmeanlogUsigma)
+            revdiffUgamma <- as.matrix(oldUgamma-revmeanUgamma)
+            forwdiffUgamma <- as.matrix(control$Ugamma-propmeanUgamma)
+
+            propUprior <- sum(dnorm(control$Ugamma,log=TRUE))
+            propUsigmaprior <- dnorm(control$logUsigma,control$logUsigma_priormean,control$logUsigma_priorsd,log=TRUE)
+
+            logfrac <- logfrac + propUprior - oldUprior +
+                                propUsigmaprior - oldUsigmaprior - 
+                                (0.5/h)*t(revdifflogUsigma)%*%SIGMA_logUsigmaINV%*%revdifflogUsigma + 
+                                (0.5/h)*t(forwdifflogUsigma)%*%SIGMA_logUsigmaINV%*%forwdifflogUsigma -
+                                (0.5/h)*sum(revdiffUgamma*SIGMA_UgammaINV*revdiffUgamma) + 
+                                (0.5/h)*sum(forwdiffUgamma*SIGMA_UgammaINV*forwdiffUgamma)
+        }                            
         
         ac <- min(1,exp(as.numeric(logfrac)))
         if(is.na(ac) | is.nan(ac)){
@@ -478,6 +608,10 @@ survspat <- function(   formula,
             bad <- c(bad,iteration(mcmcloop))
             warning("An acceptance probability could not be calculated for this iteration, this is likely because the spatial decay parameter was too big for this choice of 'ext'. Either increase ext, or tighten prior on spatial decay parameter. At the end of the run check $bad to see which iterations this affected. Stop the run if this problem persists.",immediate.=TRUE)
         }
+
+        
+
+        #if(iteration(mcmcloop)==2000){browser()}
         
         if(ac>runif(1)){
             beta <- newstuffpars[1:lenbeta]
@@ -485,8 +619,21 @@ survspat <- function(   formula,
             eta <- newstuffpars[(lenbeta+lenomega+1):(lenbeta+lenomega+leneta)]
             gamma <- newstuffgamma
 
+            if(control$nugget){
+                oldUprior <- propUprior
+                oldUsigmaprior <- propUsigmaprior
+            }
+
             oldlogpost <- newlogpost
         }
+        else{
+            if(control$nugget){ # revert back to old
+                control$U <- oldU
+                control$Ugamma <- oldUgamma
+                control$Usigma <- oldUsigma
+                control$logUsigma <- oldlogUsigma
+            }
+        }
         
         h <- exp(log(h) + (1/(iteration(mcmcloop)^0.5))*(ac-0.574))
         if(iteration(mcmcloop)%%100==0){
@@ -504,14 +651,44 @@ survspat <- function(   formula,
             else{
                 Ysamp <- rbind(Ysamp,as.vector(oldlogpost$Y))
             }
-            tarrec <- c(tarrec,oldlogpost$logpost)
+
+            if(control$nugget){
+                tarrec <- c(tarrec,oldlogpost$logpost + oldUprior + oldUsigmaprior)
+            }
+            else{
+                tarrec <- c(tarrec,oldlogpost$logpost)
+            }
             loglik <- c(loglik,oldlogpost$loglik)
             gammamean <- ((count-1)/count)*gammamean + (1/count)*gamma
+            if(control$nugget){
+                Umean <- ((count-1)/count)*Umean + (1/count)*control$U
+                Usigma_mean <- ((count-1)/count)*Usigma_mean + (1/count)*control$Usigma
+                if(count>1){
+                    Uvar <- ((count-2)/(count-1))*Uvar + (count/(count-1)^2)*(control$U-Umean)^2
+                    Usigma_var <- ((count-2)/(count-1))*Usigma_var + (count/(count-1)^2)*(control$Usigma-Usigma_mean)^2
+                }
+
+                if(control$savenugget){
+                    U_save <- rbind(U_save,control$U)
+                    Usigma_save <- rbind(Usigma_save,control$Usigma)
+                }
+
+            }
+            #gammasave <- rbind(gammasave,gamma)
+            indiv_loglik <- rbind(indiv_loglik,oldlogpost$indiv_loglik)
             count <- count + 1
         }
     }
     
     colnames(Ysamp) <- paste("Y",1:ncol(Ysamp),sep="")
+
+    if(control$nugget){
+        control$U <- Umean
+        control$Ugamma <- Umean / Usigma_mean 
+        control$Usigma <- Usigma_mean
+        control$logUsigma <- log(Usigma_mean)
+        warning("For models with a nugget effect, the DIC is approximate. Use WAIC instead.")
+    }
     
     # Compute DIC
     Dhat <- -2*LOGPOST(  surv=survivaldata,
@@ -572,6 +749,10 @@ survspat <- function(   formula,
     retlist$Dhat <- Dhat
     retlist$pD <- pD
     retlist$DIC <- DIC
+
+    retlist$lpd_hat <- sum(log(colMeans(exp(indiv_loglik))))
+    retlist$phat_waic <- sum(apply(indiv_loglik,2,var))
+    retlist$WAIC = -2 * (retlist$lpd_hat - retlist$phat_waic)
     
     retlist$X <- X
     retlist$survivaldata <- survivaldata
@@ -606,6 +787,19 @@ survspat <- function(   formula,
     retlist$shape <- shape
     retlist$ids <- ids    
     retlist$latentmode <- latentmode
+
+    retlist$nugget <- control$nugget
+    if(control$nugget){
+        retlist$nugget_mean <- Umean 
+        retlist$nugget_var <- Uvar
+        retlist$Usigma_mean <- Usigma_mean 
+        retlist$Usigma_var <- Usigma_var
+
+        if(control$savenugget){
+            retlist$U <- U_save
+            retlist$Usigma <- Usigma_save
+        }
+    }
     
     retlist$time.taken <- Sys.time() - start
     
